export declare class MagnumDI {
    private name;
    private dependencies;
    private children;
    readonly parent: MagnumDI;
    constructor({ name, parent }?: {
        name?: string;
        parent?: MagnumDI;
    });
    createChain(chain: string[]): MagnumDI;
    findChain(chain: string[]): MagnumDI;
    /**
     * Creates a Child instance of the current injector.
     * Calls to .get or .inject will first search this injector, if a parameter is found it will return it
     * if not it will continue up the tree until a value is found or the topmost instance is reached.
     * @return {MagnumDI}
     */
    createChild(name: string): MagnumDI;
    /**
     * Returns the parent injector, or null if it has no parent.
     *
     * @param parentName walks up the tree and returns the first injector where parentName === parent.name  defaults to global
     */
    getParent(parentName?: string): MagnumDI;
    getChild(childName: string): any;
    /**
     * Returns all keys registered for this injector.
     * @return {array}
     */
    getKeys(): string[];
    /**
     * Registers an object, string, number or function.
     * @param {string} name Name to be used in the injected function
     * @param {object|array|number|string|function} item Item to be injected.
     * @returns {*} Returns provided dependency
     */
    anything<T>(name: string, item: T): any;
    /**
     * When injected, calls as a constructor with new.
     * @param {string} name Name to be used in the injected function.
     * @param {function} fn Function to be called with new.
     * @returns {function} Returns provided function
     */
    instance<T>(name: string, fn: {
        new (): T;
    }): any;
    /**
     * When injected, calls the passed function. Returns the result of that call.
     * @param {string} name Name to be used in the injected function
     * @param {function} fn Function to be called by injector.
     * @returns {function} Returns provided function.
     */
    factory(name: string, fn: Function): any;
    /**
     * Merges supplied object into the object registered for <name>, if <name> doesn't exist it will be created.
     * @param {String} name Dependency object to modify
     * @param {Object} merge Object to merge with existing dependency
     * @returns {*} Returns provided dependency
     */
    merge(name: any, merge: any): any;
    /**
     * Returns the specified dependency.
     * @param {string} name Dependency to retrieve.
     * @returns {*|null} The named dependency item, or null.
     */
    get(name: string): any;
    /**
     * Modifies a registered anything object.
     * @param {String} name Dependency object to modify.
     * @param {Object} replacement Object to replace current registered object.
     * @returns {Object} Replaced dependency
     */
    replace(name: string, replacement: any): any;
    /**
     * Removes the specified dependency.
     * @param {string} name Registered dependency to remove.
     * @returns {boolean} The result of the operation.
     */
    unregister(name: string): any;
    /**
     * Runs the given function with args injected and with an optional context object.
     * @param {function|string} fnOrstr function - inject args and run, string - Return the named dependency.
     * @param {object} thisArg Calling context.
     * @returns {*} Returns the result of the called function.
     */
    inject(fnOrstr: any, thisArg?: any): any;
    /**
     * Checks that the param name attempting to be registered is valid JS identifier.
     * @private
     * @param name
     */
    validateName(name: any): void;
}
