"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const fp_1 = require("lodash/fp");
const Debug = require("debug");
const Dependencies_1 = require("./Dependencies");
const getFunctionParams_1 = require("./getFunctionParams");
const debug = Debug('magnum-di');
class MagnumDI {
    constructor({ name = 'global', parent = null } = {}) {
        this.name = name;
        this.dependencies = Dependencies_1.default();
        this.parent = parent || null;
        this.children = {};
        this.anything('Injector', this);
    }
    createChain(chain) {
        let clonedChain = fp_1.clone(chain);
        // if(this.name !== 'global'){
        //   throw new Error('MagnumDi.createChain can only be called on the global injector.')
        // }
        let current = this;
        while (clonedChain.length) {
            let search = clonedChain.shift();
            let child = current.getChild(search);
            // if(chain.length === 0 && child){
            //   console.log('This shoud throw')
            // }
            if (!child) {
                current = current.createChild(search);
            }
            else {
                current = child;
            }
        }
        return current;
    }
    findChain(chain) {
        let clonedChain = fp_1.clone(chain);
        // if(this.name !== 'global'){
        //   throw new Error('MagnumDi.findChain can only be called on the global injector.')
        // }
        let current = this;
        while (clonedChain.length && current !== null) {
            let search = clonedChain.shift();
            let child = current.getChild(search);
            if (!child) {
                current = null;
            }
            else {
                current = child;
            }
        }
        return current;
    }
    /**
     * Creates a Child instance of the current injector.
     * Calls to .get or .inject will first search this injector, if a parameter is found it will return it
     * if not it will continue up the tree until a value is found or the topmost instance is reached.
     * @return {MagnumDI}
     */
    createChild(name) {
        if (!name) {
            throw new Error('Child injectors must be named.');
        }
        this.children[name] = new MagnumDI({ name: name, parent: this });
        return this.children[name];
    }
    /**
     * Returns the parent injector, or null if it has no parent.
     *
     * @param parentName walks up the tree and returns the first injector where parentName === parent.name  defaults to global
     */
    getParent(parentName = 'global') {
        let parent = this.parent;
        while (parent) {
            if (parentName === parent.name) {
                return parent;
            }
            parent = parent.parent;
        }
        throw new Error(`Parent injector "${parentName}" not found.`);
    }
    getChild(childName) {
        if (!childName) {
            throw new Error('No childName argument provided.');
        }
        let found = false;
        let child = null;
        if (!fp_1.keys(this.children).length) {
            return child;
        }
        let queue = [this.children];
        while (!found && queue.length) {
            let i = queue.shift();
            fp_1.each(([key, value]) => {
                if (key === childName) {
                    found = true;
                    child = value;
                    return;
                }
                queue.push(value.children);
                // value //?
            }, fp_1.toPairs(i));
        }
        return child;
    }
    /**
     * Returns all keys registered for this injector.
     * @return {array}
     */
    getKeys() {
        return _.keys(this.dependencies.getAll());
    }
    /**
     * Registers an object, string, number or function.
     * @param {string} name Name to be used in the injected function
     * @param {object|array|number|string|function} item Item to be injected.
     * @returns {*} Returns provided dependency
     */
    anything(name, item) {
        if (!_.isString(name)) {
            throw new TypeError("First parameter of MagnumDI.anything() Must be a string.");
        }
        this.validateName(name);
        if (!item) {
            throw new TypeError(`${name}: Missing second parameter of MagnumDI.anything()`);
        }
        /**
         * Wrap a anything function in an outer function.
         * This allows the original function to be returned when requested.
         * If this was not wrapped, it would simply execute the function and return
         * whatever it returned.
         */
        if (_.isFunction(item)) {
            return this.dependencies.set(name, function () {
                return item;
            });
        }
        return this.dependencies.set(name, item);
    }
    /**
     * When injected, calls as a constructor with new.
     * @param {string} name Name to be used in the injected function.
     * @param {function} fn Function to be called with new.
     * @returns {function} Returns provided function
     */
    instance(name, fn) {
        if (!_.isString(name)) {
            throw new TypeError("First parameter of MagnumDI.instance() must be a string.");
        }
        this.validateName(name);
        if (!_.isFunction(fn)) {
            throw new TypeError(`${name}: Second parameter of MagnumDI.instance() must be a class.`);
        }
        return this.dependencies.set(name, function () {
            return new fn();
        });
    }
    /**
     * When injected, calls the passed function. Returns the result of that call.
     * @param {string} name Name to be used in the injected function
     * @param {function} fn Function to be called by injector.
     * @returns {function} Returns provided function.
     */
    factory(name, fn) {
        var self = this;
        if (!_.isString(name)) {
            throw new TypeError("First parameter of MagnumDI.factory() Must be a string.");
        }
        this.validateName(name);
        if (!_.isFunction(fn)) {
            throw new TypeError(`${name}: Second parameter of MagnumDI.factory() Must be a function.`);
        }
        return this.dependencies.set(name, function () {
            return self.inject(fn);
        });
    }
    /**
     * Merges supplied object into the object registered for <name>, if <name> doesn't exist it will be created.
     * @param {String} name Dependency object to modify
     * @param {Object} merge Object to merge with existing dependency
     * @returns {*} Returns provided dependency
     */
    merge(name, merge) {
        if (!_.isString(name)) {
            throw new TypeError("First parameter of MagnumDI.merge() must be a string.");
        }
        if (!merge) {
            throw new TypeError(`${name}: Missing second parameter of MagnumDI.merge()`);
        }
        if (_.isFunction(merge)) {
            throw new TypeError(`${name}: Merge value for MagnumDI.merge() cannot be a function.`);
        }
        let toModify = this.dependencies.get(name);
        if (!toModify) {
            return this.anything(name, merge);
        }
        if (_.isFunction(toModify)) {
            throw new Error(`${name}: MagnumDI.merge cannot merge an injectable function.`);
        }
        return _.merge(toModify, merge);
    }
    ;
    /**
     * Returns the specified dependency.
     * @param {string} name Dependency to retrieve.
     * @returns {*|null} The named dependency item, or null.
     */
    get(name) {
        let dependency = this.dependencies.get(name);
        if (_.isFunction(dependency)) {
            return dependency();
        }
        if (!dependency) {
            // if(this.parent){
            //   return this.parent.get(name)
            // }
            return (this.parent && this.parent.get(name)) || null;
        }
        debug(_.keys(this.dependencies.getAll()));
        return dependency;
    }
    ;
    /**
     * Modifies a registered anything object.
     * @param {String} name Dependency object to modify.
     * @param {Object} replacement Object to replace current registered object.
     * @returns {Object} Replaced dependency
     */
    replace(name, replacement) {
        let toModify = this.dependencies.get(name);
        if (_.isFunction(toModify)) {
            throw new Error(`${name}: MagnumDI.replace() cannot replace an injectable function`);
        }
        if (_.isFunction(replacement)) {
            throw new TypeError(`${name}: Replacement value for MagnumDI.replace() cannot be a function`);
        }
        return this.dependencies.set(name, replacement);
    }
    ;
    /**
     * Removes the specified dependency.
     * @param {string} name Registered dependency to remove.
     * @returns {boolean} The result of the operation.
     */
    unregister(name) {
        return this.dependencies.remove(name);
    }
    ;
    /**
     * Runs the given function with args injected and with an optional context object.
     * @param {function|string} fnOrstr function - inject args and run, string - Return the named dependency.
     * @param {object} thisArg Calling context.
     * @returns {*} Returns the result of the called function.
     */
    inject(fnOrstr, thisArg) {
        if (_.isString(fnOrstr)) {
            return this.get(fnOrstr);
        }
        if (!_.isFunction(fnOrstr)) {
            return null;
        }
        let argObjects = getFunctionParams_1.getFunctionParams(fnOrstr);
        let argArray = _.map(argObjects, (arg) => {
            if (arg === 'Injector') {
                return this;
            }
            return this.get(arg);
        });
        thisArg = (!thisArg) ? {} : thisArg;
        return fnOrstr.apply(thisArg, argArray);
    }
    /**
     * Checks that the param name attempting to be registered is valid JS identifier.
     * @private
     * @param name
     */
    validateName(name) {
        let noSpacesOrDashes = /\s+|-+/g;
        if (noSpacesOrDashes.test(name)) {
            throw new TypeError(`${name}: Dependency name must be a valid javascript variable, no spaces, tabs, or dashes.`);
        }
        if (this.dependencies.get(name)) {
            throw new TypeError(`Dependency "${name}" is already registered.`);
        }
    }
    ;
}
exports.MagnumDI = MagnumDI;
// export = MagnumDI
//# sourceMappingURL=MagnumDI.js.map